/******************************************************************************
 * DatabaseImpl.java - Implement the BuckoFIBS database API.
 * $Id: DatabaseImpl.java,v 1.2 2009/02/21 18:00:01 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright 2009 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: DatabaseImpl.java,v $
 * Revision 1.2  2009/02/21 18:00:01  dick
 * Add getAllRatingsByDate()
 *
 * Revision 1.1  2009/02/20 10:24:33  dick
 * Implement the BuckoFIBS database API.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 */

package com.buckosoft.fibs.BuckoFIBS.db;

import java.util.List;

import org.hibernate.Session;

import com.buckosoft.fibs.domain.FinishedMatch;
import com.buckosoft.fibs.domain.Player;

/** Implement the BuckoFIBS database API.
 * @author Dick Balaska
 * @since 2009/02/18
 * @version $Revision: 1.2 $ <br> $Date: 2009/02/21 18:00:01 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/BuckoFIBS/db/DatabaseImpl.java">cvs DatabaseImpl.java</a>
 */
public class DatabaseImpl implements Database {
	private boolean	virgin = true;
	
	private	List<FinishedMatch>	finishedMatches;

	public DatabaseImpl() {
	}

	private void initDB() {
		if (virgin)
			HibernateUtil.maybeCreateNewDatabase();
		virgin = false;
	}

	/* (non-Javadoc)
	 * @see com.buckosoft.fibs.BuckoFIBS.db.Database#store(com.buckosoft.fibs.domain.Player)
	 */
	@Override
	public void store(Player player) {
		initDB();
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();
		session.save(player);
		session.getTransaction().commit();

	}

	/* (non-Javadoc)
	 * @see com.buckosoft.fibs.BuckoFIBS.db.Database#getPlayer(int)
	 */
	@Override
	public Player getPlayer(int id) {
		initDB();
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();
		Player p = null;
		try {
			p = (Player)session.createQuery("from Player where id = ?")
					.setInteger(0, id).uniqueResult();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		session.getTransaction().commit();
		return p;
	}

	/* (non-Javadoc)
	 * @see com.buckosoft.fibs.BuckoFIBS.db.Database#getPlayer(java.lang.String)
	 */
	@Override
	public Player getPlayer(String name) {
		initDB();
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();
		Player p = null;
		try {
			p = (Player)session.createQuery("from Player where name = ?")
					.setString(0, name).uniqueResult();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		session.getTransaction().commit();
		return p;
	}

	/* (non-Javadoc)
	 * @see com.buckosoft.fibs.BuckoFIBS.db.Database#store(com.buckosoft.fibs.domain.FinishedMatch)
	 */
	@Override
	public void store(FinishedMatch finishedMatch) {
		initDB();
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();
		session.save(finishedMatch);
		session.getTransaction().commit();
		this.finishedMatches = null;
	}

	@SuppressWarnings("unchecked")
	private	void getFinishedMatches() {
		initDB();
		Session session = HibernateUtil.getSessionFactory().getCurrentSession();
		session.beginTransaction();
		List<?> list = session.createQuery("from FinishedMatch ORDER BY MatchDate").list();
		finishedMatches = (List<FinishedMatch>) list;
		session.getTransaction().commit();
	}

	/* (non-Javadoc)
	 * @see com.buckosoft.fibs.BuckoFIBS.db.Database#getAllRatingsByDate()
	 */
	@Override
	public double[] getAllRatingsByDate() {
		if (this.finishedMatches == null)
			getFinishedMatches();
		double[] dd = new double[this.finishedMatches.size()];
		//Iterator<FinishedMatch> iter = this.finishedMatches.iterator();
		int i = 0;
		for (FinishedMatch fm : this.finishedMatches) {
			dd[i++] = fm.getRatingAfterMatch();
		}
		//while (iter.hasNext()) {
			
		//}
		return dd;
	}


}
