/******************************************************************************
 * PlayerTableModel.java - The data model behind the list of Players
 * $Id: PlayerTableModel.java,v 1.6 2009/02/24 08:04:53 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright 2008 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: PlayerTableModel.java,v $
 * Revision 1.6  2009/02/24 08:04:53  dick
 * Add the hostName as a column.  Javadoc.
 *
 * Revision 1.5  2009/02/14 15:46:36  dick
 * BuckoFIBS is released under the GNU license.
 *
 * Revision 1.4  2009/02/05 05:58:18  dick
 * Get rid of the toodoo markers.
 *
 * Revision 1.3  2009/02/01 21:29:23  dick
 * Column 0 is now a Player, not a name.
 *
 * Revision 1.2  2009/01/28 22:32:13  dick
 * Javadoc.
 *
 * Revision 1.1  2009/01/28 19:39:44  dick
 * package com.buckosoft.fibs.gui.playerList becomes com.buckosoft.fibs.BuckoFIBS.gui.playerList.
 *
 * Revision 1.7  2009/01/28 08:31:33  dick
 * Add removeAll to clean the list.
 *
 * Revision 1.6  2009/01/22 05:07:16  dick
 * Rating and exp are class Double and Integer.
 *
 * Revision 1.5  2009/01/22 04:42:58  dick
 * Move the "playing" test up top, it is the highest priority in the test.
 * Add getColumnWidth().
 *
 * Revision 1.4  2009/01/09 07:19:39  dick
 * Turn off debug.
 *
 * Revision 1.3  2008/12/11 10:01:40  dick
 * Display the correct status.
 *
 * Revision 1.2  2008/12/09 19:41:34  dick
 * getPlayer(String) searches the list for this player.
 *
 * Revision 1.1  2008/04/02 23:22:53  dick
 * Move playerList to it's own package.
 *
 * Revision 1.3  2008/04/01 04:22:15  dick
 * Handle player logging out.
 * Fire event when player changes.
 *
 * Revision 1.2  2008/03/31 07:11:39  dick
 * Start fleshing out the table.
 *
 * Revision 1.1  2008/03/29 08:57:43  dick
 * BuckoFIBS skeleton checkin.
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.BuckoFIBS.gui.playerList;

import java.util.LinkedList;
import java.util.ListIterator;

import javax.swing.table.AbstractTableModel;

import com.buckosoft.fibs.domain.Player;

/** The data model behind the list of Players
 * @author Dick Balaska
 * @since 2008/03/29
 * @version $Revision: 1.6 $ <br> $Date: 2009/02/24 08:04:53 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/BuckoFIBS/gui/playerList/PlayerTableModel.java">cvs PlayerTableModel.java</a>
 */
public class PlayerTableModel extends AbstractTableModel {
	private	final static boolean DEBUG = false;
	private static final long serialVersionUID = 1L;

	private	PlayerColumns	_playerColumns = new PlayerColumns();
	private	LinkedList<Player>	players = new LinkedList<Player>();
	

	/** Update or add this player to the table.
	 * @param player The player who's state has changed.
	 */
	public void playerChanged(Player player) {
		ListIterator<Player> liter = players.listIterator();
		int row = -1;
		while (liter.hasNext()) {
			row++;
			Player p = liter.next();
			if (p.getName().equals(player.getName())) {
				liter.set(player);
				this.fireTableRowsUpdated(row, row);
				if (DEBUG)
					System.out.println("playerChanged: " + player.getName() + " - " + player.isReady());
				return;
			}
		}
		players.add(player);
		if (DEBUG)
			System.out.println("playerChanged: " + player.getName());
		this.fireTableRowsInserted(players.size()-1, players.size()-1);
	}

	/** Remove this player from the table.
	 * @param playerName The name of the player who left.
	 */
	public void playerGone(String playerName) {
		ListIterator<Player> liter = players.listIterator();
		int row = -1;
		while (liter.hasNext()) {
			row++;
			Player p = liter.next();
			if (p.getName().equals(playerName)) {
				liter.remove();
				this.fireTableRowsDeleted(row, row);
			}
		}
	}
	
	/** Remove all of the Players in the list.
	 */
	public void removeAll() {
		players.clear();
		this.fireTableDataChanged();		
	}

	/** Return the Player in the table that matches this name.
	 * @param playerName The name to query
	 * @return The Player found or null if no match
	 */
	public Player getPlayer(String playerName) {
		for (Player p : players) {
			if (p.getName().equals(playerName))
				return(p);
		}
		return(null);
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getColumnClass(int)
	 */
	public Class<?> getColumnClass(int arg0) {
		try {
			if (arg0 == 0)
				return(Class.forName("com.buckosoft.fibs.domain.Player"));
			if (arg0 == 2)
				return Class.forName("java.lang.Double");
			if (arg0 == 3)
				return Class.forName("java.lang.Integer");
			return Class.forName("java.lang.String");
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return(null);
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getColumnCount()
	 */
	public int getColumnCount() {
		return _playerColumns.getColumns().size();
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getColumnName(int)
	 */
	public String getColumnName(int arg0) {
		return _playerColumns.getColumns().get(arg0).getName();
	}

	public int getColumnWidth(int arg0) {
		return _playerColumns.getColumns().get(arg0).getWidth();
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getRowCount()
	 */
	public int getRowCount() {
		return this.players.size();
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#getValueAt(int, int)
	 */
	public Object getValueAt(int row, int col) {
		Player p = players.get(row);
		switch (col) {
		case 0:
			return(p);
		case 1:
			if (!p.getOpponent().isEmpty())
				return("Playing");
			if (p.isReady())
				return("Ready");
			if (p.isAway())
				return("Away");
			return("Online");
		case 2:
			return(p.getRating());
		case 3:
			return(p.getExperience());
		case 4:
			return(p.getHostName());
		}
		return null;
	}

	/** All cells are non-editable.
	 * @return false
	 * @see javax.swing.table.TableModel#isCellEditable(int, int)
	 */
	public boolean isCellEditable(int arg0, int arg1) {
		return false;
	}

	/* (non-Javadoc)
	 * @see javax.swing.table.TableModel#setValueAt(java.lang.Object, int, int)
	 */
	public void setValueAt(Object arg0, int arg1, int arg2) {
	}

}
